package de.ozgcloud.apilib.common.command.grpc;

import de.itvsh.ozg.pluto.grpc.command.CommandServiceGrpc.CommandServiceBlockingStub;
import de.itvsh.ozg.pluto.grpc.command.GrpcGetCommandRequest;
import de.ozgcloud.apilib.common.callcontext.OzgCloudCallContextAttachingInterceptor;
import de.ozgcloud.apilib.common.callcontext.OzgCloudCallContextProvider;
import de.ozgcloud.apilib.common.command.OzgCloudCommand;
import de.ozgcloud.apilib.common.command.OzgCloudCommandId;
import de.ozgcloud.apilib.common.command.OzgCloudCommandService;
import lombok.RequiredArgsConstructor;
import net.devh.boot.grpc.client.inject.GrpcClient;

@RequiredArgsConstructor
public class GrpcOzgCloudCommandService implements OzgCloudCommandService {

	private static final int WAIT_TIME_MS = 500;

	@GrpcClient("command-manager")
	private final CommandServiceBlockingStub commandServiceStub;

	private final CommandMapper mapper;

	private final OzgCloudCallContextProvider contextProvider;

	@Override
	public OzgCloudCommand createAndWaitUntilDone(OzgCloudCommand commandToCreate) {
		return waitUntilDone(create(commandToCreate));
	}

	OzgCloudCommand create(OzgCloudCommand commandToCreate) {
		var created = getCommandServiceStub().createCommand(mapper.buildCreateCommandRequest(commandToCreate));
		return mapper.fromGrpc(created.getCommand());
	}

	OzgCloudCommand waitUntilDone(OzgCloudCommand commandToWaitFor) {
		var command = commandToWaitFor;
//		TODO timeout
		while (!command.getStatus().isFinalState()) {
			synchronized (this) {
				try {
					wait(WAIT_TIME_MS);
					command = reloadCommand(command.getId());
				} catch (InterruptedException e) {
					Thread.currentThread().interrupt();
				}
			}
		}
		return command;
	}

	private OzgCloudCommand reloadCommand(OzgCloudCommandId commandId) {
		GrpcGetCommandRequest request = GrpcGetCommandRequest.newBuilder().setId(commandId.toString()).build();
		return mapper.fromGrpc(getCommandServiceStub().getCommand(request));
	}

	CommandServiceBlockingStub getCommandServiceStub() {
		return commandServiceStub.withInterceptors(new OzgCloudCallContextAttachingInterceptor(contextProvider));
	}
}
