package de.ozgcloud.apilib.common.command.grpc;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.itvsh.ozg.pluto.grpc.command.CommandServiceGrpc.CommandServiceBlockingStub;
import de.ozgcloud.apilib.common.command.OzgCloudCommand;

class GrpcOzgCloudCommandServiceTest {

	@Spy
	@InjectMocks
	private GrpcOzgCloudCommandService service;

	@Mock
	private CommandServiceBlockingStub serviceStub;
	@Mock
	private CommandMapper mapper;

	@Nested
	class TestCreateAndWaitUntilDone {
		private OzgCloudCommand command = OzgCloudCommandTestFactory.create();

		@Test
		void shouldCallCreate() {
			var created = OzgCloudCommandTestFactory.create();
			doReturn(created).when(service).create(any());

			service.createAndWaitUntilDone(command);

			verify(service).create(command);
		}

		@Test
		void shouldWaitUntilDone() {
			var created = OzgCloudCommandTestFactory.create();
			doReturn(created).when(service).create(any());

			service.createAndWaitUntilDone(command);

			verify(service).waitUntilDone(created);
		}

		@Nested
		class Create {

			@BeforeEach
			void init() {
				when(serviceStub.withInterceptors(any())).thenReturn(serviceStub);
				when(serviceStub.createCommand(any())).thenReturn(GrpcCommandResponseTestFactory.create());
			}

			@Test
			void shouldBuildRequest() {
				service.create(command);

				verify(mapper).buildCreateCommandRequest(command);
			}

			@Test
			void shouldCallStub() {
				var request = GrpcCreateCommandRequestTestFactory.create();
				doReturn(request).when(mapper).buildCreateCommandRequest(any());

				service.create(OzgCloudCommandTestFactory.create());

				verify(serviceStub).createCommand(request);
			}

			@Test
			void shouldMapCommand() {
				service.create(command);

				verify(mapper).fromGrpc(GrpcCommandTestFactory.create());
			}

			@Test
			void shouldReturnResult() {
				var expected = OzgCloudCommandTestFactory.create();
				when(mapper.fromGrpc(any())).thenReturn(expected);

				var returned = service.create(command);

				assertThat(returned).isSameAs(expected);
			}
		}
	}

}
